//
// Created by aelwazir on 10/15/22.
//

#ifndef ROCMTOOLS_ROCMTOOLS_DEVICE_PROFILE_H
#define ROCMTOOLS_ROCMTOOLS_DEVICE_PROFILE_H

#include "rocmtools.h"

// TODO(aelwazir): deice profiling Error codes to be defined
/**
 * Create a device profiling session
 *
 * A device profiling session allows the user to profile the GPU device
 * for counters irrespective of the running applications on the GPU.
 * This is different from application profiling. device profiling session
 * doesn't care about the host running processes and threads. It directly
 * provides low level profiling information.
 *
 * @param[in] counter_names The names of the counters to be collected.
 * @param[in] num_counters The number of counters specifief to be collected
 * @param[out] session_id Pointer to the created session id.
 * @param[in] cpu_index index of the cpu to be used
 * @param[in] gpu_index index of the gpu to be used
 * @retval ::ROCMTOOLS_STATUS_SUCCESS The function has been executed
 * successfully.
 */
ROCMTOOLS_API rocmtools_status_t rocmtools_device_profiling_session_create(
    const char** counter_names, uint64_t num_counters,
    rocmtools_session_id_t* session_id, int cpu_index,
    int gpu_index) ROCMTOOLS_VERSION_1_5;

/**
 * Start the device profiling session that was created previously.
 * This will enable the GPU device to start incrementing counters
 *
 * @param[in] session_id session id of the session to start
 * @retval ::ROCMTOOLS_STATUS_SUCCESS The function has been executed
 * successfully.
 */
ROCMTOOLS_API rocmtools_status_t rocmtools_device_profiling_session_start(
    rocmtools_session_id_t session_id) ROCMTOOLS_VERSION_1_5;

/**
 * Poll the device profiling session to read counters from the GPU device.
 * This will read out the values of the counters from the GPU device at the
 * specific instant when this API is called. This is a thread-blocking call.
 * Any thread that calls this API will have to wait until
 * the counter values are being read out.
 *
 * @param[in] session_id session id of the session to start
 * @param[out] data records of counter data read out from device
 * @retval ::ROCMTOOLS_STATUS_SUCCESS The function has been executed
 * successfully.
 */
ROCMTOOLS_API rocmtools_status_t rocmtools_device_profiling_session_poll(
    rocmtools_session_id_t session_id,
    rocmtools_record_counter_instance_t* data) ROCMTOOLS_VERSION_1_5;

/**
 * Stop the device profiling session that was created previously.
 * This will inform the GPU device to stop counters collection.
 *
 * @param[in] session_id session id of the session to start
 * @retval ::ROCMTOOLS_STATUS_SUCCESS The function has been executed
 * successfully.
 */
ROCMTOOLS_API rocmtools_status_t rocmtools_device_profiling_session_stop(
    rocmtools_session_id_t session_id) ROCMTOOLS_VERSION_1_5;

/**
 * Destry the device profiling session that was created previously.
 *
 * @param[in] session_id session id of the session to start
 * @retval ::ROCMTOOLS_STATUS_SUCCESS The function has been executed
 * successfully.
 */
ROCMTOOLS_API rocmtools_status_t rocmtools_device_profiling_session_destroy(
    rocmtools_session_id_t session_id) ROCMTOOLS_VERSION_1_5;

#endif  // ROCMTOOLS_ROCMTOOLS_DEVICE_PROFILE_H
